import React, { useEffect, useState } from "react"
import { version } from "../../../package.json"
import { decodeUnicode } from "../../utils/helper"
//import { FormattedMessage } from "react-intl"
import { Card, Statistic, Row, Col, Button, notification } from "antd"
import { CloudUploadOutlined, CloudDownloadOutlined } from "@ant-design/icons"

const Dashboard = () => {
	const [current, setCurrent] = useState("")
	const [packageContent, setPackageContent] = useState([])
	const [sha, setSha] = useState("")

	const handleGitGetPackage = async () => {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/package.json`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})
			// Return
			if (fetchGitApi) {
				return fetchGitApi
			}
		} catch (error) {
			console.log("Error handleGitGetPackage ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	const handleGitUpdatePackage = async (contentStr, fileSha) => {
		try {
			const data = {
				message: "Update package.json",
				content: btoa(contentStr),
				sha: fileSha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/package.json`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})
			// Return
			if (fetchGitApi) {
				return true
			}
		} catch (error) {
			console.log("Error handleGitUpdatePackage!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}

	const handleUpdate = () => {
		if (current.indexOf(version) === -1) {
			if (packageContent) {
				if (
					packageContent.dependencies[`@maad-dev/gatsby-theme-backend`] ===
					current
				) {
					packageContent.dependencies[
						`@maad-dev/gatsby-theme-backend`
					] = `^${version}`
					const packageStr = JSON.stringify(packageContent)
					// Process
					handleGitUpdatePackage(packageStr, sha).then((updateFile) => {
						if (updateFile) {
							notification["success"]({
								message: "Success!",
								description: "Updated package.json successfully.",
								duration: 0,
							})
						} else {
							let errMsg = "No status"
							if (updateFile === 404) {
								errMsg = "Status: 404 Not Found"
							} else if (updateFile === 409) {
								errMsg = "Status: 409 Conflict"
							} else if (updateFile === 422) {
								errMsg = "Status: 422 Unprocessable Entity"
							}
							notification["error"]({
								message: "Error!",
								description: errMsg,
								duration: 0,
							})
						}
					})
				}
			}
		}
	}

	useEffect(() => {
		async function fetchOauth() {
			await handleGitGetPackage().then((getFile) => {
				if (getFile !== false) {
					const decodeContent = decodeUnicode(getFile.content)
					if (decodeContent) {
						const packageJson = JSON.parse(decodeContent)
						if (packageJson) {
							const dependencies = packageJson.dependencies
							const currentVersion =
								dependencies[`@maad-dev/gatsby-theme-backend`]
							setCurrent(currentVersion)
							setSha(getFile.sha)
							setPackageContent(packageJson)
						} else {
							setCurrent("api error")
						}
					}
				}
			})
		}

		fetchOauth()
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [])

	return (
		<Row gutter={[16, 16]}>
			<Col xs={24} sm={12}>
				<Card>
					<Statistic
						title="Latest backend version"
						value={version}
						valueStyle={{ color: "#f56a00" }}
						prefix={<CloudUploadOutlined />}
					/>
				</Card>
			</Col>
			<Col xs={24} sm={12}>
				<Card>
					<Statistic
						title="Current backend version"
						value={current}
						valueStyle={{ color: "#000000d9" }}
						prefix={<CloudDownloadOutlined />}
					/>
				</Card>
			</Col>
			<Col span={24}>
				<Card title="Get all new features">
					<Button
						type="primary"
						onClick={handleUpdate}
						disabled={current.indexOf(version) !== -1}
					>
						Upgrade Version
					</Button>
				</Card>
			</Col>
		</Row>
	)
}

export default Dashboard
